@@ -2,31 +2,29 @@
 
 """This module implements the base CCDPROC functions"""
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
+import math
 import numbers
+import logging
 
 import numpy as np
-import math
-from astropy.extern import six
+from scipy import ndimage
+
 from astropy.units.quantity import Quantity
 from astropy import units as u
 from astropy.modeling import fitting
 from astropy import stats
 from astropy.nddata import utils as nddata_utils
-from astropy.nddata import StdDevUncertainty
+from astropy.nddata import StdDevUncertainty, CCDData
 from astropy.wcs.utils import proj_plane_pixel_area
 from astropy.utils import deprecated
 import astropy  # To get the version.
 
-from scipy import ndimage
-
-from .ccddata import CCDData
 from .utils.slices import slice_from_string
 from .log_meta import log_to_metadata
 from .extern.bitfield import bitfield_to_boolean_mask as _bitfield_to_boolean_mask
 
+logger = logging.getLogger(__name__)
+
 __all__ = ['background_deviation_box', 'background_deviation_filter',
            'ccd_process', 'cosmicray_median', 'cosmicray_lacosmic',
            'create_deviation', 'flat_correct', 'gain_correct', 'rebin',
@@ -182,7 +180,7 @@ def ccd_process(ccd, oscan=None, trim=None, error=False, master_bias=None,
 
         >>> import numpy as np
         >>> from astropy import units as u
-        >>> from ccdproc import CCDData
+        >>> from astropy.nddata import CCDData
         >>> from ccdproc import ccd_process
         >>> ccd = CCDData(np.ones([100, 100]), unit=u.adu)
         >>> nccd = ccd_process(ccd, oscan='[1:10,1:100]',
@@ -197,7 +195,7 @@ def ccd_process(ccd, oscan=None, trim=None, error=False, master_bias=None,
         nccd = subtract_overscan(nccd, overscan=oscan,
                                  median=oscan_median,
                                  model=oscan_model)
-    elif isinstance(oscan, six.string_types):
+    elif isinstance(oscan, str):
         nccd = subtract_overscan(nccd, fits_section=oscan,
                                  median=oscan_median,
                                  model=oscan_model)
@@ -207,7 +205,7 @@ def ccd_process(ccd, oscan=None, trim=None, error=False, master_bias=None,
         raise TypeError('oscan is not None, a string, or CCDData object.')
 
     # apply the trim correction
-    if isinstance(trim, six.string_types):
+    if isinstance(trim, str):
         nccd = trim_image(nccd, fits_section=trim)
     elif trim is None:
         pass
@@ -275,7 +273,7 @@ def ccd_process(ccd, oscan=None, trim=None, error=False, master_bias=None,
 
 
 @log_to_metadata
-def create_deviation(ccd_data, gain=None, readnoise=None):
+def create_deviation(ccd_data, gain=None, readnoise=None, disregard_nan=False):
     """
     Create a uncertainty frame. The function will update the uncertainty
     plane which gives the standard deviation for the data. Gain is used in
@@ -298,6 +296,10 @@ def create_deviation(ccd_data, gain=None, readnoise=None):
         Read noise per pixel.
         Default is ``None``.
 
+    disregard_nan: boolean
+        If ``True``, any value of nan in the output array will be replaced by
+        the readnoise.
+
     {log}
 
     Raises
@@ -333,9 +335,20 @@ def create_deviation(ccd_data, gain=None, readnoise=None):
     gain_value = float(gain / gain.unit)
     readnoise_value = float(readnoise / readnoise.unit)
 
-    var = (gain_value * ccd_data.data + readnoise_value ** 2) ** 0.5
-    ccd = ccd_data.copy()
+    # remove values that might be negative or treat as nan
+    data = gain_value * ccd_data.data
+    mask = (data < 0)
+    if disregard_nan:
+        data[mask] = 0
+    else:
+        data[mask] = np.nan
+        logging.warning('Negative values in array will be replaced with nan')
+
+    # calculate the deviation
+    var = (data + readnoise_value ** 2) ** 0.5
+
     # ensure uncertainty and image data have same unit
+    ccd = ccd_data.copy()
     var /= gain_value
     ccd.uncertainty = StdDevUncertainty(var)
     return ccd
@@ -443,7 +456,7 @@ def subtract_overscan(ccd, overscan=None, overscan_axis=1, fits_section=None,
         raise TypeError('overscan is not a CCDData object.')
 
     if (fits_section is not None and
-            not isinstance(fits_section, six.string_types)):
+            not isinstance(fits_section, str)):
         raise TypeError('overscan is not a string.')
 
     if fits_section is not None:
@@ -532,7 +545,7 @@ def trim_image(ccd, fits_section=None):
     ``arr1``, not a copy.
     """
     if (fits_section is not None and
-            not isinstance(fits_section, six.string_types)):
+            not isinstance(fits_section, str)):
         raise TypeError("fits_section must be a string.")
     trimmed = ccd.copy()
     if fits_section:
@@ -622,6 +635,10 @@ def subtract_dark(ccd, master, dark_exposure=None, data_exposure=None,
     result : `~astropy.nddata.CCDData`
         Dark-subtracted image.
     """
+    if ccd.shape != master.shape:
+        err_str = "operands could not be subtracted with shapes {} {}".format(ccd.shape, master.shape)
+        raise ValueError(err_str)
+
     if not (isinstance(ccd, CCDData) and isinstance(master, CCDData)):
         raise TypeError("ccd and master must both be CCDData objects.")
 
@@ -854,6 +871,10 @@ def transform_image(ccd, transform_func, **kwargs):
         mask = transform_func(nccd.mask, **kwargs)
         nccd.mask = mask > 0
 
+    if nccd.wcs is not None:
+        warn = 'WCS information may be incorrect as no transformation was applied to it'
+        logging.warning(warn)
+
     return nccd
 
 
@@ -896,6 +917,7 @@ def wcs_project(ccd, target_wcs, target_shape=None, order='bilinear'):
     ccd : `~astropy.nddata.CCDData`
         A transformed CCDData object.
     """
+    from astropy.nddata.ccddata import _generate_wcs_and_update_header
     from reproject import reproject_interp
 
     if not (ccd.wcs.is_celestial and target_wcs.is_celestial):
@@ -935,9 +957,12 @@ def wcs_project(ccd, target_wcs, target_shape=None, order='bilinear'):
     if not output_mask.any():
         output_mask = None
 
+    # If there are any wcs keywords in the header, remove them
+    hdr, _ = _generate_wcs_and_update_header(ccd.header)
+
     nccd = CCDData(area_ratio * projected_image_raw, wcs=target_wcs,
                    mask=output_mask,
-                   header=ccd.header, unit=ccd.unit)
+                   header=hdr, unit=ccd.unit)
 
     return nccd
 
@@ -1730,8 +1755,8 @@ def ccdmask(ratio, findbadcolumns=False, byblocks=False, ncmed=7, nlmed=7,
     if byblocks:
         nlinesblock = int(math.ceil(nlines / nlsig))
         ncolsblock = int(math.ceil(ncols / ncsig))
-        for i in six.moves.range(nlinesblock):
-            for j in six.moves.range(ncolsblock):
+        for i in range(nlinesblock):
+            for j in range(ncolsblock):
                 l1 = i * nlsig
                 l2 = min((i + 1) * nlsig, nlines)
                 c1 = j * ncsig
@@ -1762,10 +1787,10 @@ def ccdmask(ratio, findbadcolumns=False, byblocks=False, ncmed=7, nlmed=7,
         # Loop through columns and look for short segments (<ngood pixels long)
         # which are unmasked, but are surrounded by masked pixels and mask them
         # under the assumption that the column region is bad.
-        for col in six.moves.range(0, ncols):
-            for line in six.moves.range(0, nlines - ngood - 1):
+        for col in range(0, ncols):
+            for line in range(0, nlines - ngood - 1):
                 if mask[line, col]:
-                    for i in six.moves.range(2, ngood + 2):
+                    for i in range(2, ngood + 2):
                         lend = line + i
                         if (mask[lend, col] and
                                 not np.all(mask[line:lend + 1, col])):
@@ -1815,30 +1840,30 @@ def bitfield_to_boolean_mask(bitfield, ignore_bits=0, flip_bits=None):
         >>> import ccdproc
         >>> import numpy as np
         >>> ccdproc.bitfield_to_boolean_mask(np.arange(8))
-        array([False,  True,  True,  True,  True,  True,  True,  True], dtype=bool)
+        array([False,  True,  True,  True,  True,  True,  True,  True]...)
 
     To ignore all bit flags ``ignore_bits=None`` can be used::
 
         >>> ccdproc.bitfield_to_boolean_mask(np.arange(8), ignore_bits=None)
-        array([False, False, False, False, False, False, False, False], dtype=bool)
+        array([False, False, False, False, False, False, False, False]...)
 
     To ignore only specific bit flags one can use a ``list`` of bits flags to
     ignore::
 
         >>> ccdproc.bitfield_to_boolean_mask(np.arange(8), ignore_bits=[1, 4])
-        array([False, False,  True,  True, False, False,  True,  True], dtype=bool)
+        array([False, False,  True,  True, False, False,  True,  True]...)
 
     There are some equivalent ways::
 
         >>> # pass in the sum of the "ignore_bits" directly
         >>> ccdproc.bitfield_to_boolean_mask(np.arange(8), ignore_bits=5)  # 1 + 4
-        array([False, False,  True,  True, False, False,  True,  True], dtype=bool)
+        array([False, False,  True,  True, False, False,  True,  True]...)
         >>> # use a comma seperated string of integers
         >>> ccdproc.bitfield_to_boolean_mask(np.arange(8), ignore_bits='1, 4')
-        array([False, False,  True,  True, False, False,  True,  True], dtype=bool)
+        array([False, False,  True,  True, False, False,  True,  True]...)
         >>> # use a + seperated string of integers
         >>> ccdproc.bitfield_to_boolean_mask(np.arange(8), ignore_bits='1+4')
-        array([False, False,  True,  True, False, False,  True,  True], dtype=bool)
+        array([False, False,  True,  True, False, False,  True,  True]...)
 
     Instead of directly specifying the **bits flags to ignore** one can also
     pass in the **only bits that shouldn't be ignored** by prepending a ``~``
@@ -1847,14 +1872,14 @@ def bitfield_to_boolean_mask(bitfield, ignore_bits=0, flip_bits=None):
 
         >>> # ignore all bit flags except the one for 2.
         >>> ccdproc.bitfield_to_boolean_mask(np.arange(8), ignore_bits='~(2)')
-        array([False, False,  True,  True, False, False,  True,  True], dtype=bool)
+        array([False, False,  True,  True, False, False,  True,  True]...)
         >>> # ignore all bit flags except the one for 1, 8 and 32.
         >>> ccdproc.bitfield_to_boolean_mask(np.arange(8), ignore_bits='~(1, 8, 32)')
-        array([False,  True, False,  True, False,  True, False,  True], dtype=bool)
+        array([False,  True, False,  True, False,  True, False,  True]...)
 
         >>> # Equivalent for a list using flip_bits.
         >>> ccdproc.bitfield_to_boolean_mask(np.arange(8), ignore_bits=[1, 8, 32], flip_bits=True)
-        array([False,  True, False,  True, False,  True, False,  True], dtype=bool)
+        array([False,  True, False,  True, False,  True, False,  True]...)
 
     """
     return _bitfield_to_boolean_mask(
@@ -1862,7 +1887,7 @@ def bitfield_to_boolean_mask(bitfield, ignore_bits=0, flip_bits=None):
         good_mask_value=False, dtype=bool)
 
 
-class Keyword(object):
+class Keyword:
     """
     """
     def __init__(self, name, unit=None, value=None):
@@ -1889,7 +1914,7 @@ class Keyword(object):
         elif isinstance(value, Quantity):
             self._unit = value.unit
             self._value = value
-        elif isinstance(value, six.string_types):
+        elif isinstance(value, str):
             if self.unit is not None:
                 raise ValueError("keyword with a unit cannot have a "
                                  "string value.")
