@@ -1,11 +1,10 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-from __future__ import (print_function, division, absolute_import,
-                        unicode_literals)
-
 from collections import OrderedDict
 import fnmatch
 from os import listdir, path
+import re
+
 import logging
 
 import numpy as np
@@ -13,13 +12,13 @@ import numpy.ma as ma
 
 from astropy.table import Table, MaskedColumn
 import astropy.io.fits as fits
-from astropy.extern import six
 from astropy.utils import minversion
 
 import warnings
 from astropy.utils.exceptions import AstropyUserWarning
 
 from .ccddata import fits_ccddata_reader, _recognized_fits_file_extensions
+from .tests.pytest_fixtures import directory_for_testing
 
 logger = logging.getLogger(__name__)
 
@@ -29,7 +28,7 @@ __doctest_skip__ = ['*']
 _ASTROPY_LT_1_3 = not minversion("astropy", "1.3")
 
 
-class ImageFileCollection(object):
+class ImageFileCollection:
     """
     Representation of a collection of image files.
 
@@ -52,18 +51,16 @@ class ImageFileCollection(object):
         columns. Default value is '*' unless ``info_file`` is specified.
         Default is ``None``.
 
-    info_file : str or None, optional
-        Path to file that contains a table of information about FITS files.
-        In this case the keywords are set to the names of the columns of the
-        ``info_file`` unless ``keywords`` is explicitly set to a different
-        list.
-        Default is ``None``.
-
-        .. deprecated:: 1.3
+    find_fits_by_reading: bool, optional
+        If ``True``, read each file in location to check whether the file is a
+        FITS file and include it in the collection based on that, rather than
+        by file name. Compressed files, e.g. image.fits.gz, will **NOT** be
+        properly detected. *Will be ignored if `filenames` is not ``None``.*
 
     filenames: str, list of str, or None, optional
         List of the names of FITS files which will be added to the collection.
-        The filenames are assumed to be in ``location``.
+        The filenames may either be in ``location`` or the name can be a
+        relative or absolute path to the file.
         Default is ``None``.
 
     glob_include: str or None, optional
@@ -78,9 +75,9 @@ class ImageFileCollection(object):
         easily select subsets of files in the target directory.
         Default is ``None``.
 
-     ext: str or int, optional
-         The extension from which the header and data will be read in all files.
-         Default is ``0``.
+    ext: str or int, optional
+        The extension from which the header and data will be read in all
+        files.Default is ``0``.
 
     Raises
     ------
@@ -88,13 +85,10 @@ class ImageFileCollection(object):
         Raised if keywords are set to a combination of '*' and any other
         value.
     """
-    def __init__(self, location=None, keywords=None, info_file=None,
+
+    def __init__(self, location=None, keywords=None,
+                 find_fits_by_reading=False,
                  filenames=None, glob_include=None, glob_exclude=None, ext=0):
-
-        if info_file is not None:
-            warnings.warn("The 'info_file' argument is deprecated and will be "
-                          "removed in a future version", DeprecationWarning)
-
         # Include or exclude files from the collection based on glob pattern
         # matching - has to go above call to _get_files()
         if glob_exclude is not None:
@@ -105,41 +99,24 @@ class ImageFileCollection(object):
             glob_include = str(glob_include)
         self._glob_include = glob_include
 
-        self._location = location
+        if location is not None:
+            self._location = location
+        else:
+            self._location = ''
+
+        self._find_fits_by_reading = find_fits_by_reading
+
         self._filenames = filenames
         self._files = []
-        self._info_file = info_file
-        if location:
-            self._files = self._get_files()
+        self._files = self._get_files()
 
         if self._files == []:
             warnings.warn("no FITS files in the collection.",
                           AstropyUserWarning)
         self._summary = {}
         if keywords is None:
-            if info_file is not None:
-                # Default to empty list so that keywords will be populated
-                # from table columns names.
-                keywords = []
-            else:
-                # Otherwise use all keywords.
-                keywords = '*'
-        if info_file is not None:
-            try:
-                info_path = path.join(self.location, info_file)
-            except (AttributeError, TypeError):
-                info_path = info_file
-            try:
-                self._summary = Table.read(info_path, format='ascii',
-                                           delimiter=',')
-                self._summary = Table(self._summary, masked=True)
-            except IOError:
-                if location:
-                    logger.warning('unable to open table file %s, will try '
-                                   'initializing from location instead.',
-                                   info_path)
-                else:
-                    raise
+            # Use all keywords.
+            keywords = '*'
 
         # Used internally to keep track of whether the user asked for all
         # keywords or a specific list. The keywords setter takes care of
@@ -163,11 +140,6 @@ class ImageFileCollection(object):
         else:
             kw = "keywords={!r}".format(self.keywords[1:])
 
-        if self._info_file is None:
-            infofile = ''
-        else:
-            infofile = "info_file={!r}".format(self._info_file)
-
         if self.glob_exclude is None:
             glob_exclude = ''
         else:
@@ -188,7 +160,7 @@ class ImageFileCollection(object):
         else:
             filenames = "filenames={}".format(self._filenames)
 
-        params = [location, kw, infofile, filenames, glob_include, glob_exclude, ext]
+        params = [location, kw, filenames, glob_include, glob_exclude, ext]
         params = ', '.join([p for p in params if p])
 
         str_repr = "{self.__class__.__name__}({params})".format(
@@ -221,23 +193,6 @@ class ImageFileCollection(object):
         """
         return self._summary
 
-    @property
-    def summary_info(self):
-        """
-        `~astropy.table.Table` of values of FITS keywords for files in the
-        collection.
-
-        Each keyword is a column heading. In addition, there is a column
-        called 'file' that contains the name of the FITS file. The directory
-        is not included as part of that name.
-
-        .. deprecated:: 0.4
-        """
-        warnings.warn('"summary_info" is deprecated and will be removed in '
-                      'a future version. Use the "summary" attribute instead.',
-                      AstropyUserWarning)
-        return self._summary
-
     @property
     def location(self):
         """
@@ -359,11 +314,10 @@ class ImageFileCollection(object):
         if keyword not in self.keywords:
             raise ValueError(
                 'keyword %s is not in the current summary' % keyword)
-
         if unique:
-            return list(set(self.summary[keyword]))
+            return list(set(self.summary[keyword].tolist()))
         else:
-            return list(self.summary[keyword])
+            return self.summary[keyword].tolist()
 
     def files_filtered(self, **kwd):
         """Determine files whose keywords have listed values.
@@ -375,6 +329,12 @@ class ImageFileCollection(object):
             contains not just the filename, but the full path to each file.
             Default is ``False``.
 
+        regex_match : bool, keyword-only
+            If ``True``, then string values in the ``**kwd`` dictionary are
+            treated as regular expression patterns and matching is done by
+            regular expression search. The search is always
+            **case insensitive**.
+
         **kwd :
             ``**kwd`` is dict of keywords and values the files must have.
             The value '*' represents any value.
@@ -404,10 +364,11 @@ class ImageFileCollection(object):
 
         Notes
         -----
-        Value comparison is case *insensitive* for strings.
+        Value comparison is case *insensitive* for strings, whether matching
+        exactly or matching with regular expressions.
         """
         # force a copy by explicitly converting to a list
-        current_file_mask = list(self.summary['file'].mask)
+        current_file_mask = self.summary['file'].mask.tolist()
 
         include_path = kwd.pop('include_path', False)
 
@@ -416,7 +377,7 @@ class ImageFileCollection(object):
         self.summary['file'].mask = current_file_mask
         if include_path:
             filtered_files = [path.join(self._location, f)
-                              for f in filtered_files]
+                              for f in filtered_files.tolist()]
         return filtered_files
 
     def refresh(self):
@@ -441,7 +402,36 @@ class ImageFileCollection(object):
         """
         if len(self._summary) > 0:
             self._summary.sort(keys)
-            self._files = list(self.summary['file'])
+            self._files = self.summary['file'].tolist()
+
+    def filter(self, **kwd):
+        """
+        Create a new collection by filtering the current collection.
+
+        Parameters
+        ----------
+
+        regex_match : bool, keyword-only
+            If ``True``, then string values in the ``**kwd`` dictionary are
+            treated as regular expression patterns and matching is done by
+            regular expression search. The search is always
+            **case insensitive**.
+
+        **kwd :
+            ``**kwd`` is dict of keywords and values the files must have.
+            The value '*' represents any value.
+            A missing keyword is indicated by value ''.
+
+        Returns
+        -------
+
+        `ImageFileCollection`
+            A new collection with the files matched by the arguments
+            to filter.
+        """
+        files = self.files_filtered(include_path=True, **kwd)
+        return ImageFileCollection(filenames=files,
+                                   keywords=self.keywords)
 
     def _get_files(self):
         """ Helper method which checks whether ``files`` should be set
@@ -454,7 +444,7 @@ class ImageFileCollection(object):
         """
         files = []
         if self._filenames:
-            if isinstance(self._filenames, six.string_types):
+            if isinstance(self._filenames, str):
                 files.append(self._filenames)
             else:
                 files = self._filenames
@@ -513,12 +503,19 @@ class ImageFileCollection(object):
 
         assert 'file' not in h
 
+        if self.location:
+            # We have a location and can reconstruct the path using it
+            name_for_file_column = path.basename(file_name)
+        else:
+            # No location, so use whatever path the user passed in
+            name_for_file_column = file_name
+
         # Try opening header before this so that file name is only added if
         # file is valid FITS
         try:
-            summary['file'].append(path.basename(file_name))
+            summary['file'].append(name_for_file_column)
         except KeyError:
-            summary['file'] = [path.basename(file_name)]
+            summary['file'] = [name_for_file_column]
 
         missing_in_this_file = [k for k in summary if (k not in h and
                                                        k != 'file')]
@@ -527,7 +524,7 @@ class ImageFileCollection(object):
                             'history': []}
 
         alreadyencountered = set()
-        for k, v in six.iteritems(h):
+        for k, v in h.items():
             if k == '':
                 continue
 
@@ -556,7 +553,7 @@ class ImageFileCollection(object):
 
             _add_val_to_dict(k, v, summary, n_previous, missing_marker)
 
-        for k, v in six.iteritems(multi_entry_keys):
+        for k, v in multi_entry_keys.items():
             if v:
                 joined = ','.join(v)
                 _add_val_to_dict(k, joined, summary, n_previous,
@@ -609,7 +606,7 @@ class ImageFileCollection(object):
         summary_dict = None
         missing_marker = None
 
-        for file_name in file_name_column:
+        for file_name in file_name_column.tolist():
             file_path = path.join(self.location, file_name)
             try:
                 # Note: summary_dict is an OrderedDict, so should preserve
@@ -625,8 +622,8 @@ class ImageFileCollection(object):
         summary_table = Table(summary_dict, masked=True)
 
         for column in summary_table.colnames:
-            summary_table[column].mask = [v is missing_marker
-                                          for v in summary_table[column]]
+            summary_table[column].mask = [
+                v is missing_marker for v in summary_table[column].tolist()]
 
         self._set_column_name_case_to_match_keywords(header_keys,
                                                      summary_table)
@@ -656,6 +653,16 @@ class ImageFileCollection(object):
         """
         Find files whose keywords have given values.
 
+        Parameters
+        ----------
+
+        match_regex : bool, optional
+            If ``True`` match string values by using a regular expression
+            search instead of equality. Default value is ``False``.
+
+        The remaining arguments are keyword/value pairs specifying the
+        values to match.
+
         `**kwd` is list of keywords and values the files must have.
 
         The value '*' represents any value.
@@ -667,9 +674,11 @@ class ImageFileCollection(object):
             >>> collection = ImageFileCollection('test/data', keywords=keys)
             >>> collection.files_filtered(imagetyp='LIGHT', filter='R')
             >>> collection.files_filtered(imagetyp='*', filter='')
+            >>> collection.files_filtered(imagetyp='bias|filter', regex_match=True)
 
         NOTE: Value comparison is case *insensitive* for strings.
         """
+        regex_match = kwd.pop('regex_match', False)
         keywords = kwd.keys()
         values = kwd.values()
 
@@ -690,19 +699,29 @@ class ImageFileCollection(object):
             if value == '*':
                 have_this_value = value_not_missing
             elif value is not None:
-                if isinstance(value, six.string_types):
+                if isinstance(value, str):
                     # need to loop explicitly over array rather than using
                     # where to correctly do string comparison.
                     have_this_value = np.zeros(len(use_info), dtype=bool)
-                    for idx, file_key_value in enumerate(use_info[key]):
+
+                    # We are going to do a regex match no matter what.
+                    if regex_match:
+                        pattern = re.compile(value,
+                                             flags=re.IGNORECASE)
+                    else:
+                        # This pattern matches the prior behavior.
+                        pattern = re.compile('^' + value + '$',
+                                             flags=re.IGNORECASE)
+
+                    for idx, file_key_value in enumerate(use_info[key].tolist()):
                         if value_not_missing[idx]:
                             try:
                                 value_matches = (
-                                    file_key_value.lower() == value.lower())
-                            except AttributeError:
+                                    pattern.search(file_key_value) is not None)
+                            except TypeError:
                                 # In case we're dealing with an object column
                                 # there could be values other than strings in it
-                                # so it could fail with an AttributeError.
+                                # so it could fail with an TypeError.
                                 value_matches = False
                         else:
                             value_matches = False
@@ -734,7 +753,7 @@ class ImageFileCollection(object):
 
         Parameters
         ----------
-        extension : list of str or None, optional
+        extensions : list of str or None, optional
             List of filename extensions that are FITS files. Default is
             ``['fit', 'fits', 'fts']``.
             Default is ``None``.
@@ -752,14 +771,25 @@ class ImageFileCollection(object):
 
         full_extensions = extensions or list(_recognized_fits_file_extensions)
 
+        # The common compressed fits image .fz is supported using ext=1 when calling ImageFileCollection
         if compressed:
-            with_gz = [extension + '.gz' for extension in full_extensions]
-            full_extensions.extend(with_gz)
+            for comp in ['.gz', '.bz2', '.Z', '.zip', '.fz']:
+                with_comp = [extension + comp for extension in full_extensions]
+                full_extensions.extend(with_comp)
 
         all_files = listdir(self.location)
         files = []
-        for extension in full_extensions:
-            files.extend(fnmatch.filter(all_files, '*' + extension))
+        if not self._find_fits_by_reading:
+            for extension in full_extensions:
+                files.extend(fnmatch.filter(all_files, '*' + extension))
+        else:
+            for infile in all_files:
+                inpath = path.join(self.location, infile)
+                with open(inpath, 'rb') as fp:
+                    # Hmm, first argument to is_fits is not actually used in
+                    # that function. *shrug*
+                    if fits.connect.is_fits('just some junk', infile, fp):
+                        files.append(infile)
 
         files.sort()
         return files
@@ -825,6 +855,13 @@ class ImageFileCollection(object):
             See `~astropy.nddata.fits_ccddata_reader` for a complete list of
             parameters that can be passed through ``ccd_kwargs``.
 
+
+        regex_match : bool, keyword-only
+            If ``True``, then string values in the ``**kwd`` dictionary are
+            treated as regular expression patterns and matching is done by
+            regular expression search. The search is always
+            **case insensitive**.
+
         **kwd :
             Any additional keywords are used to filter the items returned; see
             `files_filtered` examples for details.
@@ -931,7 +968,7 @@ class ImageFileCollection(object):
         """
         Full path to each file.
         """
-        unmasked_files = self.summary['file'].compressed()
+        unmasked_files = self.summary['file'].compressed().tolist()
         return [path.join(self.location, file_) for file_ in unmasked_files]
 
     def headers(self, do_not_scale_image_data=True, **kwd):
@@ -964,3 +1001,14 @@ class ImageFileCollection(object):
         return self._generator('ccd', ccd_kwargs=ccd_kwargs, **kwd)
     ccds.__doc__ = _generator.__doc__.format(
         name='CCDData', default_scaling='True', return_type='astropy.nddata.CCDData')
+
+
+def sample_directory_with_files():
+    """
+    Returns the path to the small sample directory used
+    in the tests of ``ImageFileCollection``. Primarily intended
+    for use in the doctests.
+    """
+
+    n_test, tmpdir = directory_for_testing()
+    return tmpdir
